package interfaces;

import core.Position;
import core.TileMove;
import core.TileType;
import exceptions.IllegalBoardSettingsException;
import exceptions.IllegalMoveException;

/**
 * Interface for controlling the board.
 *
 * @author Fernando Sanchez Villaamil
 */
public interface Board {

    /**
     * Sets the whole board.
     *
     * @param tileMatrix A matrix of tiles. The first dimension gives the
     * position on the x-axis, the second on the y-axis.
     * @param freeTile The tile which can be inserted in the next move.
     * @param objectivePositions The positions of the objectives.
     * @param playerPositions The positions of the players.
     * @param unmovableTiles The position of the tiles that are unmovable. This
     * array can be <code>null</code>, which means, that there are no unmovable 
     * tiles.
     * @throws IllegalBoardSettingsException Thrown if the information does not
     * represent a constructible board.
     */
    public void setBoard(TileType[][] tileMatrix, TileType freeTile,
	    Position[] objectivePositions, Position[] playerPositions,
	    Position[] unmovableTiles) throws IllegalBoardSettingsException;

    /**
     * Perform the first part of a move, where the free tile is inserted into
     * the board.
     *
     * @param move An object representing the position of the insertion and the
     * tile type to be inserted.
     * @throws IllegalMoveException Thrown if the move is not valid.
     */
    public void insertTile(TileMove move) throws IllegalMoveException;

    /**
     * Second part of a move, where the player changes position.
     *
     * @param player The player to be moved.
     * @param pos The position to which it must me moved.
     * @throws IllegalMoveException Thrown if the given player can not move to
     * the given position.
     */
    public void movePlayer(int player, Position pos) throws IllegalMoveException;

    public Tile getTile(int posX, int posY) throws IllegalMoveException;

    public Tile getFreeTile();

    public int getWidth();

    public int getHeight();
}
