package test;

import core.Position;
import core.TileMove;
import core.TileType;
import exceptions.IllegalBoardSettingsException;
import exceptions.IllegalMoveException;
import interfaces.Board;
import java.util.ArrayList;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 * A hopefully thorough test of the functionality of the board.
 * 
 * @author Fernando Sanchez Villaamil, Felix Reidl
 */
public abstract class TestBoard {

    public abstract Board instantiateBoard();

    @Test
    public final void testSettingOfBoard() {
	// 3x4 matrix
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.LB, TileType.BLANK, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});

	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	objectives.add(new Position(0, 0));
	objectives.add(new Position(0, 0));
	objectives.add(new Position(3, 2));

	PosList players = new PosList();
	players.add(new Position(1, 1));
	players.add(new Position(1, 1));
	players.add(new Position(3, 2));

	PosList unmovableTiles = new PosList();
	unmovableTiles.add(new Position(1, 0));
	unmovableTiles.add(new Position(2, 2));

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(),
		players.toArray(), unmovableTiles.toArray());

	assertEquals(4, board.getWidth());
	assertEquals(3, board.getHeight());

	assertEquals(TileType.LB, board.getTile(0, 0).getType());
	assertEquals(TileType.BLANK, board.getTile(1, 0).getType());
	assertEquals(TileType.LR, board.getTile(2, 0).getType());
	assertEquals(TileType.LRB, board.getTile(3, 0).getType());
	assertEquals(TileType.LT, board.getTile(0, 1).getType());
	assertEquals(TileType.LTB, board.getTile(1, 1).getType());
	assertEquals(TileType.LTR, board.getTile(2, 1).getType());
	assertEquals(TileType.LTRB, board.getTile(3, 1).getType());
	assertEquals(TileType.RB, board.getTile(0, 2).getType());
	assertEquals(TileType.TB, board.getTile(1, 2).getType());
	assertEquals(TileType.TR, board.getTile(2, 2).getType());
	assertEquals(TileType.TRB, board.getTile(3, 2).getType());

	assertEquals(TileType.LTRB, board.getFreeTile().getType());

	assertTrue(board.getTile(0, 0).getObjectives().contains(0));
	assertTrue(board.getTile(0, 0).getObjectives().contains(1));
	assertTrue(board.getTile(1, 0).getObjectives().isEmpty());
	assertTrue(board.getTile(2, 0).getObjectives().isEmpty());
	assertTrue(board.getTile(3, 0).getObjectives().isEmpty());
	assertTrue(board.getTile(0, 1).getObjectives().isEmpty());
	assertTrue(board.getTile(1, 1).getObjectives().isEmpty());
	assertTrue(board.getTile(2, 1).getObjectives().isEmpty());
	assertTrue(board.getTile(3, 1).getObjectives().isEmpty());
	assertTrue(board.getTile(0, 2).getObjectives().isEmpty());
	assertTrue(board.getTile(1, 2).getObjectives().isEmpty());
	assertTrue(board.getTile(2, 2).getObjectives().isEmpty());
	assertTrue(board.getTile(3, 2).getObjectives().contains(2));

	assertTrue(board.getTile(0, 0).getPlayers().isEmpty());
	assertTrue(board.getTile(1, 0).getPlayers().isEmpty());
	assertTrue(board.getTile(2, 0).getPlayers().isEmpty());
	assertTrue(board.getTile(3, 0).getPlayers().isEmpty());
	assertTrue(board.getTile(0, 1).getPlayers().isEmpty());
	assertTrue(board.getTile(1, 1).getPlayers().contains(0));
	assertTrue(board.getTile(1, 1).getPlayers().contains(1));
	assertTrue(board.getTile(2, 1).getPlayers().isEmpty());
	assertTrue(board.getTile(3, 1).getPlayers().isEmpty());
	assertTrue(board.getTile(0, 2).getPlayers().isEmpty());
	assertTrue(board.getTile(1, 2).getPlayers().isEmpty());
	assertTrue(board.getTile(2, 2).getPlayers().isEmpty());
	assertTrue(board.getTile(3, 2).getPlayers().contains(2));

	assertTrue(board.getTile(0, 0).isMovable());
	assertFalse(board.getTile(1, 0).isMovable());
	assertTrue(board.getTile(2, 0).isMovable());
	assertTrue(board.getTile(3, 0).isMovable());
	assertTrue(board.getTile(0, 1).isMovable());
	assertTrue(board.getTile(1, 1).isMovable());
	assertTrue(board.getTile(2, 1).isMovable());
	assertTrue(board.getTile(3, 1).isMovable());
	assertTrue(board.getTile(0, 2).isMovable());
	assertTrue(board.getTile(1, 2).isMovable());
	assertFalse(board.getTile(2, 2).isMovable());
	assertTrue(board.getTile(3, 2).isMovable());

    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testNullFreeTile() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB},
		    {TileType.LT, TileType.LTB}});
	TileType freeTile = null;

	PosList objectives = new PosList();

	PosList players = new PosList();

	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	assertTrue(board.getTile(0, 0).getPlayers().isEmpty());
	assertTrue(board.getTile(1, 0).getPlayers().isEmpty());
	assertTrue(board.getTile(0, 1).getPlayers().isEmpty());
	assertTrue(board.getTile(1, 1).getPlayers().isEmpty());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testNullMatrix() {
	TileType[][] tileMatrix = null;
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testZeroWidth() {
	TileType[][] tileMatrix = {};// Zero width
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testZeroHeight() {
	TileType[][] tileMatrix = {{}, {}, {}, {}}; // Zero height
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testPlayerOnBlank() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.LRB, TileType.BLANK},
		    {TileType.LT, TileType.LTB}});
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	PosList players = new PosList();
	players.add(new Position(1, 0));
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testObjectiveOnBlank() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.LRB, TileType.BLANK},
		    {TileType.BLANK, TileType.LTB}});
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	objectives.add(new Position(1, 0));
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testNullPlayers() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB},
		    {TileType.LT, TileType.LTB}});
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), null,
		unmovableTiles.toArray());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testNullObjectives() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB},
		    {TileType.LT, TileType.LTB}});
	TileType freeTile = TileType.LTRB;

	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, null, players.toArray(),
		unmovableTiles.toArray());
    }

    @Test
    public void testNullUnmovableTiles() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB},
		    {TileType.LT, TileType.LTB}});
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	PosList players = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		null);

	assertTrue(board.getTile(0, 0).isMovable());
	assertTrue(board.getTile(1, 0).isMovable());
	assertTrue(board.getTile(0, 1).isMovable());
	assertTrue(board.getTile(1, 1).isMovable());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testInvalidObjectivePosition() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	objectives.add(new Position(1, 1));
	objectives.add(new Position(1, 1));
	objectives.add(new Position(3, 3)); //Invalid position

	PosList players = new PosList();

	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());
    }

    @Test(expected = IllegalBoardSettingsException.class)
    public void testInvalidPlayerPosition() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();

	PosList players = new PosList();
	players.add(new Position(1, 1));
	players.add(new Position(1, 1));
	players.add(new Position(3, 3)); //Invalid position

	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());
    }

    @Test
    public final void testMakingMoveFromLeft() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.LB, TileType.LR, TileType.BLANK, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LTRB;

	PosList objectives = new PosList();
	objectives.add(new Position(0, 0));
	objectives.add(new Position(0, 0));
	objectives.add(new Position(1, 0));
	objectives.add(new Position(3, 0));

	PosList players = new PosList();
	players.add(new Position(0, 0));
	players.add(new Position(0, 0));
	players.add(new Position(1, 0));
	players.add(new Position(3, 0));

	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.left(0, freeTile);
	board.insertTile(move);

	assertEquals(TileType.LRB, board.getFreeTile().getType());
	assertEquals(TileType.LTRB, board.getTile(0, 0).getType());
	assertEquals(TileType.LB, board.getTile(1, 0).getType());
	assertEquals(TileType.LR, board.getTile(2, 0).getType());
	assertEquals(TileType.BLANK, board.getTile(3, 0).getType());

	assertTrue(board.getFreeTile().getObjectives().contains(3));
	assertTrue(board.getTile(0, 0).getObjectives().isEmpty());
	assertTrue(board.getTile(1, 0).getObjectives().contains(0));
	assertTrue(board.getTile(1, 0).getObjectives().contains(1));
	assertTrue(board.getTile(2, 0).getObjectives().contains(2));
	assertTrue(board.getTile(3, 0).getObjectives().isEmpty());

	assertTrue(board.getTile(0, 0).getPlayers().contains(3));
	assertTrue(board.getTile(1, 0).getPlayers().contains(0));
	assertTrue(board.getTile(1, 0).getPlayers().contains(1));
	assertTrue(board.getTile(2, 0).getPlayers().contains(2));
	assertTrue(board.getTile(3, 0).getPlayers().isEmpty());
    }

    @Test
    public final void testMakingMoveFromTop() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.TR;

	PosList objectives = new PosList();
	objectives.add(new Position(3, 0)); // 0
	objectives.add(new Position(3, 0)); // 1
	objectives.add(new Position(3, 2)); // 2

	PosList players = new PosList();
	players.add(new Position(3, 0));
	players.add(new Position(3, 0));
	players.add(new Position(3, 1));

	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.top(3, freeTile);
	board.insertTile(move);

	assertEquals(TileType.TRB, board.getFreeTile().getType());
	assertEquals(TileType.TR, board.getTile(3, 0).getType());
	assertEquals(TileType.LRB, board.getTile(3, 1).getType());
	assertEquals(TileType.LTRB, board.getTile(3, 2).getType());

	assertTrue(board.getFreeTile().getObjectives().contains(2));
	assertTrue(board.getTile(3, 0).getObjectives().isEmpty());
	assertTrue(board.getTile(3, 1).getObjectives().contains(0));
	assertTrue(board.getTile(3, 1).getObjectives().contains(1));
	assertTrue(board.getTile(3, 2).getObjectives().isEmpty());

	assertTrue(board.getTile(3, 0).getPlayers().isEmpty());
	assertTrue(board.getTile(3, 1).getPlayers().contains(0));
	assertTrue(board.getTile(3, 1).getPlayers().contains(1));
	assertTrue(board.getTile(3, 2).getPlayers().contains(2));
    }

    @Test
    public final void testMakingMoveFromRight() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.TRB;

	PosList objectives = new PosList();
	objectives.add(new Position(0, 1)); // 0
	objectives.add(new Position(0, 1)); // 1
	objectives.add(new Position(1, 1)); // 2
	objectives.add(new Position(3, 1)); // 3

	PosList players = new PosList();
	players.add(new Position(0, 1));
	players.add(new Position(0, 1));
	players.add(new Position(1, 1));
	players.add(new Position(3, 1));

	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.right(1, freeTile);
	board.insertTile(move);

	assertEquals(TileType.LT, board.getFreeTile().getType());
	assertEquals(TileType.TRB, board.getTile(3, 1).getType());
	assertEquals(TileType.LTRB, board.getTile(2, 1).getType());
	assertEquals(TileType.LTR, board.getTile(1, 1).getType());
	assertEquals(TileType.LTB, board.getTile(0, 1).getType());

	assertTrue(board.getFreeTile().getObjectives().contains(0));
	assertTrue(board.getFreeTile().getObjectives().contains(1));

	assertTrue(board.getTile(0, 1).getObjectives().contains(2));
	assertTrue(board.getTile(1, 1).getObjectives().isEmpty());
	assertTrue(board.getTile(2, 1).getObjectives().contains(3));
	assertTrue(board.getTile(3, 1).getObjectives().isEmpty());

	assertTrue(board.getTile(0, 1).getPlayers().contains(2));
	assertTrue(board.getTile(1, 1).getPlayers().isEmpty());
	assertTrue(board.getTile(2, 1).getPlayers().contains(3));
	assertTrue(board.getTile(3, 1).getPlayers().contains(0));
	assertTrue(board.getTile(3, 1).getPlayers().contains(1));
    }

    @Test
    public final void testMakingMoveFromBottom() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LR;

	PosList objectives = new PosList();
	objectives.add(new Position(2, 0)); // 0
	objectives.add(new Position(2, 0)); // 1
	objectives.add(new Position(2, 2)); // 2

	PosList players = new PosList();
	players.add(new Position(2, 0));
	players.add(new Position(2, 0));
	players.add(new Position(2, 1));

	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.bottom(2, freeTile);
	board.insertTile(move);

	assertEquals(TileType.LR, board.getFreeTile().getType());
	assertEquals(TileType.LTR, board.getTile(2, 0).getType());
	assertEquals(TileType.TR, board.getTile(2, 1).getType());
	assertEquals(TileType.LR, board.getTile(2, 2).getType());

	assertTrue(board.getFreeTile().getObjectives().contains(0));
	assertTrue(board.getFreeTile().getObjectives().contains(1));
	assertTrue(board.getTile(2, 0).getObjectives().isEmpty());
	assertTrue(board.getTile(2, 1).getObjectives().contains(2));

	assertTrue(board.getTile(2, 0).getPlayers().contains(2));
	assertTrue(board.getTile(2, 1).getPlayers().isEmpty());
	assertTrue(board.getTile(2, 2).getPlayers().contains(0));
	assertTrue(board.getTile(2, 2).getPlayers().contains(1));
    }

    @Test(expected = IllegalMoveException.class)
    public final void testTryingToMoveUnmovableColumn() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.BLANK;

	PosList objectives = new PosList();
	PosList players = new PosList();

	PosList unmovableTiles = new PosList();
	unmovableTiles.add(new Position(1, 1));

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.bottom(1, freeTile);
	board.insertTile(move);
    }

    @Test(expected = IllegalMoveException.class)
    public final void testTryingToMoveUnmovableRow() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.BLANK;

	PosList objectives = new PosList();
	PosList players = new PosList();

	PosList unmovableTiles = new PosList();
	unmovableTiles.add(new Position(1, 1));

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.left(1, freeTile);
	board.insertTile(move);
    }

    @Test(expected = IllegalMoveException.class)
    public final void testTryingToMoveNonExistingColumn() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.BLANK;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.top(6, freeTile);
	board.insertTile(move);
    }

    @Test(expected = IllegalMoveException.class)
    public final void testTryingToMoveNonExistingRow() {
	TileType[][] tileMatrix = transpose(new TileType[][]{{TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.BLANK;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.right(5, freeTile);
	board.insertTile(move);
    }

    /**
     * This test tries all rotations of TileType.LR.
     */
    @Test
    public final void testTryingToInsertRotatedTile1() {
	TileType[][] tileMatrix = transpose(new TileType[][]{{TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LR;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.right(0, TileType.TB);
	board.insertTile(move);
    }

    /**
     * This test tries all rotations of TileType.LB.
     */
    @Test
    public final void testTryingToInsertRotatedTile2() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
                    {TileType.LB, TileType.LB, TileType.LB, TileType.LB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LB;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.right(0, TileType.LT);
	board.insertTile(move);
        
	move = TileMove.right(0, TileType.RB);
	board.insertTile(move);
        
	move = TileMove.right(0, TileType.TR);
	board.insertTile(move);
    }

    /**
     * This test tries all rotations of TileType.LRB.
     */
    @Test
    public final void testTryingToInsertRotatedTile3() {
	TileType[][] tileMatrix = transpose(new TileType[][]{{TileType.LRB, TileType.LRB, TileType.LRB, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LRB;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.right(0, TileType.LTB);
	board.insertTile(move);
    }

    @Test(expected = IllegalMoveException.class)
    public final void testTryingToInsertIllegalTile1() {
	TileType[][] tileMatrix = transpose(new TileType[][]{{TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.BLANK;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.right(0, TileType.LB);
	board.insertTile(move);
    }

    @Test(expected = IllegalMoveException.class)
    public final void testTryingToInsertIllegalTile2() {
	TileType[][] tileMatrix = transpose(new TileType[][]{{TileType.BLANK, TileType.LB, TileType.LR, TileType.LRB},
		    {TileType.LT, TileType.LTB, TileType.LTR, TileType.LTRB},
		    {TileType.RB, TileType.TB, TileType.TR, TileType.TRB}});
	TileType freeTile = TileType.LR;

	PosList objectives = new PosList();
	PosList players = new PosList();
	PosList unmovableTiles = new PosList();

	Board board = this.instantiateBoard();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), players.toArray(),
		unmovableTiles.toArray());

	TileMove move = TileMove.right(0, TileType.RB);
	board.insertTile(move);
    }

    /**
     * A spill-over occurs when the tile that becomes free after the insertion
     * move contains players.
     */
    @Test
    public final void testSpillOver() {
	TileType[][] tileMatrix = transpose(new TileType[][]{
		    {TileType.LR, TileType.LR, TileType.LR, TileType.LRB, TileType.LT}});
	TileType freeTile = TileType.BLANK;

	// No players
	Board board = this.instantiateBoard();
        PosList players = new PosList();
        PosList objectives = new PosList();
	board.setBoard(tileMatrix, freeTile, objectives.toArray(), 
                players.toArray(), null);
	board.insertTile(TileMove.left(0, TileType.BLANK));

	assertEquals(board.getFreeTile().getType(), TileType.LT);
	assertEquals(board.getTile(0, 0).getType(), TileType.BLANK);

	// With players
	board = this.instantiateBoard();
	players = new PosList();
	players.add(new Position(4, 0)); // 0
	players.add(new Position(3, 0)); // 1
	players.add(new Position(2, 0)); // 2

	board.setBoard(tileMatrix, freeTile, objectives.toArray(), 
                players.toArray(), null);
	board.insertTile(TileMove.left(0, TileType.BLANK));

	assertEquals(board.getFreeTile().getType(), TileType.LRB);
	assertEquals(board.getTile(0, 0).getType(), TileType.LT);
	assertEquals(board.getTile(1, 0).getType(), TileType.BLANK);
	assertEquals(board.getTile(4, 0).getType(), TileType.LR);

	assertTrue(board.getTile(0, 0).getPlayers().size() == 2);
	assertTrue(board.getTile(0, 0).getPlayers().contains(0));
	assertTrue(board.getTile(0, 0).getPlayers().contains(1));
	assertTrue(board.getTile(4, 0).getPlayers().contains(2));
    }

    public static TileType[][] transpose(TileType[][] matrix) {
	TileType[][] res = new TileType[matrix[0].length][matrix.length];

	for (int x = 0; x < matrix.length; x++) {
	    for (int y = 0; y < matrix[0].length; y++) {
		res[y][x] = matrix[x][y];
	    }
	}

	return res;
    }

    private static class PosList extends ArrayList<Position> {
	@Override
	public Position[] toArray() {
	    return super.toArray(new Position[size()]);
	}
    }
}
